@@ -1,24 +1,5 @@
-# MIT License
-#
-# Copyright (c) 2018-2019 Red Hat, Inc.
-
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in all
-# copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
+# Copyright Contributors to the Packit project.
+# SPDX-License-Identifier: MIT
 
 import datetime
 import functools
@@ -36,12 +17,23 @@ from ogr.abstract import (
     CommitStatus,
 )
 from ogr.constant import DEFAULT_RO_PREFIX_STRING
-from ogr.deprecation import deprecate_and_set_removal
 
 
 def log_output(
     text: str, default_prefix: str = DEFAULT_RO_PREFIX_STRING, namespace: str = __name__
 ) -> None:
+    """
+    Logs output.
+
+    Args:
+        text: Text message to be logged.
+        default_prefix: Prefix of the log message.
+
+            Defaults to `DEFAULT_RO_PREFIX_STRING`.
+        namespace: Namespace where the message comes from.
+
+            Defaults to `__name__`.
+    """
     logger = logging.getLogger(namespace)
     logger.warning(f"{default_prefix} {text}")
 
@@ -53,14 +45,25 @@ def if_readonly(
     log_message: str = "",
 ) -> Any:
     """
-    Decorator to log  function and ovewrite return value of object methods
-    Ignore function name as first parameter and ignore every other parameters
+    Decorator to log function and ovewrite return value of object methods.
+    Ignore function name as first parameter and ignore every other parameters.
 
-    :param return_value: returned Any value if given, return_function has higher prio if set
-    :param return_function: return function and give there parameters also
-           original caller object return_function(self, *args, **kwargs)
-    :param log_message: str string to put to logger output
-    :return: Any type what is expected that function or return value returns
+    Args:
+        return_value: Returned value if given, `return_function`
+            has higher priority if set.
+
+            Defaults to `None`.
+        return_function: Returned function with applies
+            arguments and original caller.
+
+            Defaults to `None`.
+        log_message: String to be put to logger output.
+
+            Defaults to `""`.
+
+    Returns:
+        Any value that is expected to be returned from the function call or the
+        specificied return value.
     """
 
     def decorator_readonly(func):
@@ -161,29 +164,6 @@ class GitProjectReadOnly:
     author = "ReadOnlyAuthor"
     url = "url://ReadOnlyURL"
 
-    @classmethod
-    @deprecate_and_set_removal(
-        since="0.9.0",
-        remove_in="0.14.0 (or 1.0.0 if it comes sooner)",
-        message="Use create_pr",
-    )
-    def pr_create(
-        cls,
-        original_object: Any,
-        title: str,
-        body: str,
-        target_branch: str,
-        source_branch: str,
-        fork_username: str = None,
-    ) -> "PullRequest":
-        return GitProjectReadOnly.create_pr(
-            original_object=original_object,
-            title=title,
-            body=body,
-            target_branch=target_branch,
-            source_branch=source_branch,
-        )
-
     @classmethod
     def create_pr(
         cls,
@@ -216,7 +196,7 @@ class GitProjectReadOnly:
         filename: str = None,
         row: int = None,
     ) -> "PRComment":
-        pull_request = original_object.get_pr_info(pr_id)
+        pull_request = original_object.get_pr(pr_id)
         log_output(pull_request)
         return PRComment(
             parent=pull_request,
@@ -228,13 +208,13 @@ class GitProjectReadOnly:
 
     @classmethod
     def pr_close(cls, original_object: Any, pr_id: int) -> "PullRequest":
-        pull_request = original_object.get_pr_info(pr_id)
+        pull_request = original_object.get_pr(pr_id)
         pull_request._status = PRStatus.closed
         return pull_request
 
     @classmethod
     def pr_merge(cls, original_object: Any, pr_id: int) -> "PullRequest":
-        pull_request = original_object.get_pr_info(pr_id)
+        pull_request = original_object.get_pr(pr_id)
         pull_request._status = PRStatus.merged
         return pull_request
 
@@ -242,7 +222,7 @@ class GitProjectReadOnly:
     def issue_comment(
         cls, original_object: Any, issue_id: int, body: str
     ) -> "IssueComment":
-        issue = original_object.get_issue_info(issue_id)
+        issue = original_object.get_issue(issue_id)
         log_output(issue)
         return IssueComment(
             parent=issue,
@@ -253,7 +233,9 @@ class GitProjectReadOnly:
         )
 
     @classmethod
-    def fork_create(cls, original_object: Any) -> "GitProject":
+    def fork_create(
+        cls, original_object: Any, namespace: Optional[str] = None
+    ) -> "GitProject":
         return original_object
 
     @classmethod
