@@ -1,24 +1,5 @@
-# MIT License
-#
-# Copyright (c) 2018-2019 Red Hat, Inc.
-
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in all
-# copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
+# Copyright Contributors to the Packit project.
+# SPDX-License-Identifier: MIT
 
 import datetime
 import logging
@@ -35,7 +16,6 @@ from github import UnknownObjectException
 from github.GithubException import GithubException
 from github.Repository import Repository
 from github.CommitComment import CommitComment as GithubCommitComment
-from github.GitRelease import GitRelease as PyGithubRelease
 
 from ogr.abstract import (
     Issue,
@@ -119,20 +99,16 @@ class GithubProject(BaseGitProject):
 
     @property
     def description(self) -> str:
-        """
-        Returns:
-            Project description.
-        """
         return self.github_repo.description
 
     @description.setter
     def description(self, new_description: str) -> None:
-        """
-        Args:
-            new_description: description to set for project.
-        """
         self.github_repo.edit(description=new_description)
 
+    @property
+    def has_issues(self) -> bool:
+        return self.github_repo.has_issues
+
     def _construct_fork_project(self) -> Optional["GithubProject"]:
         gh_user = self.github_instance.get_user()
         user_login = gh_user.login
@@ -145,7 +121,7 @@ class GithubProject(BaseGitProject):
                 return None
             return project
         except github.GithubException as ex:
-            logger.debug(f"Project {self.repo}/{user_login} does not exist: {ex}")
+            logger.debug(f"Project {user_login}/{self.repo} does not exist: {ex}")
             return None
 
     def exists(self) -> bool:
@@ -158,35 +134,17 @@ class GithubProject(BaseGitProject):
             raise GithubAPIException from ex
 
     def is_private(self) -> bool:
-        """
-        Is this repo private? (accessible only by users with granted access)
-
-        :return: if yes, return True
-        """
         return self.github_repo.private
 
     def is_forked(self) -> bool:
-        """
-        Is this repo forked by the authenticated user?
-
-        :return: if yes, return True
-        """
         return bool(self._construct_fork_project())
 
     @property
     def is_fork(self) -> bool:
-        """
-        Is this repository a fork?
-
-        :return: True if it is
-        """
         return self.github_repo.fork
 
     @property
     def parent(self) -> Optional["GithubProject"]:
-        """
-        Return parent project if this project is a fork, otherwise return None
-        """
         return (
             self.service.get_project_from_github_repository(self.github_repo.parent)
             if self.is_fork
@@ -204,13 +162,6 @@ class GithubProject(BaseGitProject):
         return self.github_repo.description
 
     def add_user(self, user: str, access_level: AccessLevel) -> None:
-        """
-        AccessLevel.pull => Pull
-        AccessLevel.triage => Triage
-        AccessLevel.push => Push
-        AccessLevel.admin => Admin
-        AccessLevel.maintain => Maintain
-        """
         access_dict = {
             AccessLevel.pull: "Pull",
             AccessLevel.triage: "Triage",
@@ -222,8 +173,8 @@ class GithubProject(BaseGitProject):
             invitation = self.github_repo.add_to_collaborators(
                 user, permission=access_dict[access_level]
             )
-        except Exception:
-            raise GithubAPIException("User {user} not found")
+        except Exception as ex:
+            raise GithubAPIException(f"User {user} not found") from ex
 
         if invitation is None:
             raise GithubAPIException("User already added")
@@ -232,14 +183,6 @@ class GithubProject(BaseGitProject):
         raise OperationNotSupported("Not possible on GitHub")
 
     def get_fork(self, create: bool = True) -> Optional["GithubProject"]:
-        """
-        Provide GithubProject instance of a fork of this project.
-
-        Returns None if this is a fork.
-
-        :param create: create a fork if it doesn't exist
-        :return: instance of GithubProject
-        """
         username = self.service.user.get_username()
         for fork in self.get_forks():
             if fork.github_repo.owner.login == username:
@@ -290,8 +233,10 @@ class GithubProject(BaseGitProject):
 
     def _get_collaborators_with_permission(self) -> dict:
         """
-        Get all project collaborators in dictionary with permission association
-        :return: List of usernames
+        Get all project collaborators in dictionary with permission association.
+
+        Returns:
+            Dictionary with logins of collaborators and their permission level.
         """
         collaborators = {}
         users = self.github_repo.get_collaborators()
@@ -345,6 +290,15 @@ class GithubProject(BaseGitProject):
         raise GithubAPIException(f"Tag {tag_name} was not found.")
 
     def get_tag_from_tag_name(self, tag_name: str) -> Optional[GitTag]:
+        """
+        Get a tag based on a tag name.
+
+        Args:
+            tag_name: Name of the tag.
+
+        Returns:
+            GitTag associated with the given tag name or `None`.
+        """
         all_tags = self.github_repo.get_tags()
         for tag in all_tags:
             if tag.name == tag_name:
@@ -370,15 +324,6 @@ class GithubProject(BaseGitProject):
     def commit_comment(
         self, commit: str, body: str, filename: str = None, row: int = None
     ) -> CommitComment:
-        """
-        Create comment on a commit.
-
-        :param commit: str The SHA of the commit needing a comment.
-        :param body: str The text of the comment
-        :param filename: str The relative path to the file that necessitates a comment
-        :param row: int Line index in the diff to comment on.
-        :return: CommitComment
-        """
         github_commit = self.github_repo.get_commit(commit)
         if filename and row:
             comment = github_commit.create_comment(
@@ -402,28 +347,10 @@ class GithubProject(BaseGitProject):
         context: str,
         trim: bool = False,
     ):
-        """
-        Create a status on a commit
-
-        :param commit: The SHA of the commit.
-        :param state: The state of the status.
-        :param target_url: The target URL to associate with this status.
-        :param description: A short description of the status
-        :param context: A label to differentiate this status from the status of other systems.
-        :param trim: bool Whether to trim the description in order to avoid throwing
-            github.GithubException
-        :return:
-        """
         pass
 
     @indirect(GithubCommitFlag.get)
     def get_commit_statuses(self, commit: str) -> List[CommitFlag]:
-        """
-        Get status of the commit.
-
-        :param commit: str
-        :return: [CommitFlag]
-        """
         pass
 
     @indirect(GithubCheckRun.get)
@@ -463,18 +390,17 @@ class GithubProject(BaseGitProject):
         return {"git": self.github_repo.clone_url, "ssh": self.github_repo.ssh_url}
 
     @if_readonly(return_function=GitProjectReadOnly.fork_create)
-    def fork_create(self) -> "GithubProject":
-        """
-        Fork this project using the authenticated user.
-        This may raise an exception if the fork already exists.
-
-        :return: fork GithubProject instance
-        """
-        gh_user = self.github_instance.get_user()
-        return self.service.get_project_from_github_repository(
-            gh_user.create_fork(self.github_repo)
+    def fork_create(self, namespace: Optional[str] = None) -> "GithubProject":
+        fork_repo = (
+            self.github_repo.create_fork(organization=namespace)
+            if namespace
+            else self.github_repo.create_fork()
         )
 
+        fork = self.service.get_project_from_github_repository(fork_repo)
+        logger.debug(f"Forked to {fork.namespace}/{fork.repo}")
+        return fork
+
     def change_token(self, new_token: str):
         raise OperationNotSupported
 
@@ -486,19 +412,12 @@ class GithubProject(BaseGitProject):
             ).decoded_content.decode()
         except (UnknownObjectException, GithubException) as ex:
             if ex.status == 404:
-                raise FileNotFoundError(f"File '{path}' on {ref} not found", ex)
-            raise GithubAPIException(ex)
+                raise FileNotFoundError(f"File '{path}' on {ref} not found") from ex
+            raise GithubAPIException() from ex
 
     def get_files(
         self, ref: str = None, filter_regex: str = None, recursive: bool = False
     ) -> List[str]:
-        """
-        Get a list of file paths of the repo.
-        :param ref: branch or commit (defaults to repo's default branch)
-        :param filter_regex: filter the paths with re.search
-        :param recursive: whether to return only top directory files or all files recursively
-        :return: [str]
-        """
         ref = ref or self.default_branch
         paths = []
         contents = self.github_repo.get_contents(path="", ref=ref)
@@ -525,33 +444,6 @@ class GithubProject(BaseGitProject):
 
         return paths
 
-    def _release_from_github_object(
-        self, raw_release: PyGithubRelease, git_tag: GitTag
-    ) -> GithubRelease:
-        """
-        Get ogr.abstract.Release object from github.GithubRelease
-
-        :param raw_release: GithubRelease, object from Github API
-            https://developer.github.com/v3/repos/releases/
-        :return: Release, example(type, value):
-            tag_name: str, "v1.0.0"
-            url: str, "https://api.github.com/repos/octocat/Hello-World/releases/1"
-            created_at: datetime.datetime, 2018-09-19 12:56:26
-            tarball_url: str, "https://api.github.com/repos/octocat/Hello-World/tarball/v1.0.0"
-            git_tag: GitTag
-            project: GithubProject
-            raw_release: PyGithubRelease
-        """
-        return GithubRelease(
-            tag_name=raw_release.tag_name,
-            url=raw_release.url,
-            created_at=str(raw_release.created_at),
-            tarball_url=raw_release.tarball_url,
-            git_tag=git_tag,
-            project=self,
-            raw_release=raw_release,
-        )
-
     @staticmethod
     def _commitcomment_from_github_object(
         raw_commitcoment: GithubCommitComment,
@@ -565,7 +457,9 @@ class GithubProject(BaseGitProject):
     def get_labels(self):
         """
         Get list of labels in the repository.
-        :return: [Label]
+
+        Returns:
+            List of labels in the repository.
         """
         return list(self.github_repo.get_labels())
 
@@ -573,8 +467,11 @@ class GithubProject(BaseGitProject):
         """
         Update the labels of the repository. (No deletion, only add not existing ones.)
 
-        :param labels: [str]
-        :return: int - number of added labels
+        Args:
+            labels: List of labels to be added.
+
+        Returns:
+            Number of added labels.
         """
         current_label_names = [la.name for la in list(self.github_repo.get_labels())]
         changes = 0
@@ -594,66 +491,23 @@ class GithubProject(BaseGitProject):
             return color[1:]
         return color
 
-    def _release_id_from_name(self, name) -> Optional[int]:
-        releases = self.github_repo.get_releases()
-        for release in releases:
-            if release.title == name:
-                return release.id
-        return None
-
-    def _release_id_from_tag(self, tag) -> Optional[int]:
-        releases = self.github_repo.get_releases()
-        for release in releases:
-            if release.tag_name == tag:
-                return release.id
-        return None
-
+    @indirect(GithubRelease.get)
     def get_release(self, identifier=None, name=None, tag_name=None) -> GithubRelease:
-        if tag_name:
-            identifier = self._release_id_from_tag(tag_name)
-        elif name:
-            identifier = self._release_id_from_name(name)
-        if identifier is None:
-            raise GithubAPIException("Release was not found.")
-        release = self.github_repo.get_release(id=identifier)
-        return self._release_from_github_object(
-            raw_release=release, git_tag=self.get_tag_from_tag_name(release.tag_name)
-        )
+        pass
 
+    @indirect(GithubRelease.get_latest)
     def get_latest_release(self) -> Optional[GithubRelease]:
-        try:
-            release = self.github_repo.get_latest_release()
-            return self._release_from_github_object(
-                raw_release=release,
-                git_tag=self.get_tag_from_tag_name(release.tag_name),
-            )
-        except GithubException as ex:
-            if ex.status == 404:
-                return None
-            raise GithubAPIException from ex
+        pass
 
+    @indirect(GithubRelease.get_list)
     def get_releases(self) -> List[Release]:
-        releases = self.github_repo.get_releases()
-        return [
-            self._release_from_github_object(
-                raw_release=release,
-                git_tag=self.get_tag_from_tag_name(release.tag_name),
-            )
-            for release in releases
-        ]
+        pass
 
+    @indirect(GithubRelease.create)
     def create_release(self, tag: str, name: str, message: str) -> GithubRelease:
-        created_release = self.github_repo.create_git_release(
-            tag=tag, name=name, message=message
-        )
-        return self.get_release(created_release.id)
+        pass
 
     def get_forks(self) -> List["GithubProject"]:
-        """
-        Get forks of the project.
-
-        :return: [PagureProject]
-        """
         return [
             self.service.get_project_from_github_repository(fork)
             for fork in self.github_repo.get_forks()
@@ -661,12 +515,22 @@ class GithubProject(BaseGitProject):
         ]
 
     def get_web_url(self) -> str:
-        """
-        Get web URL of the project.
-
-        :return: str
-        """
         return self.github_repo.html_url
 
     def get_tags(self) -> List["GitTag"]:
         return [GitTag(tag.name, tag.commit.sha) for tag in self.github_repo.get_tags()]
+
+    def get_sha_from_branch(self, branch: str) -> Optional[str]:
+        try:
+            return self.github_repo.get_branch(branch).commit.sha
+        except GithubException as ex:
+            if ex.status == 404:
+                return None
+            raise GithubAPIException from ex
+
+    def get_contributors(self) -> Set[str]:
+        """
+        Returns:
+            Logins of contributors to the project.
+        """
+        return set(map(lambda c: c.login, self.github_repo.get_contributors()))
