@@ -2,9 +2,10 @@
 # SPDX-License-Identifier: MIT
 
 import logging
-from typing import List, Optional, Dict, Set, Union
+from typing import Any, List, Optional, Dict, Set, Union
 
 import gitlab
+from gitlab.exceptions import GitlabGetError
 from gitlab.v4.objects import Project as GitlabObjectsProject
 
 from ogr.abstract import (
@@ -64,9 +65,6 @@ class GitlabProject(BaseGitProject):
 
     @property
     def parent(self) -> Optional["GitlabProject"]:
-        """
-        Return parent project if this project is a fork, otherwise return None
-        """
         if self.is_fork:
             parent_dict = self.gitlab_repo.attributes["forked_from_project"]
             return GitlabProject(
@@ -93,6 +91,10 @@ class GitlabProject(BaseGitProject):
             and self.service == o.service
         )
 
+    @property
+    def has_issues(self) -> bool:
+        return self.gitlab_repo.issues_enabled
+
     def _construct_fork_project(self) -> Optional["GitlabProject"]:
         user_login = self.service.user.get_username()
         try:
@@ -102,7 +104,7 @@ class GitlabProject(BaseGitProject):
             if project.gitlab_repo:
                 return project
         except Exception as ex:
-            logger.debug(f"Project {self.repo}/{user_login} does not exist: {ex}")
+            logger.debug(f"Project {user_login}/{self.repo} does not exist: {ex}")
         return None
 
     def exists(self) -> bool:
@@ -115,11 +117,6 @@ class GitlabProject(BaseGitProject):
             raise GitlabAPIException from ex
 
     def is_private(self) -> bool:
-        """
-        Is this repo private? (accessible only by users with granted access)
-
-        :return: if yes, return True
-        """
         return self.gitlab_repo.attributes["visibility"] == "private"
 
     def is_forked(self) -> bool:
@@ -130,30 +127,14 @@ class GitlabProject(BaseGitProject):
 
     @property
     def description(self) -> str:
-        """
-        Returns:
-            Project description.
-        """
         return self.gitlab_repo.attributes["description"]
 
     @description.setter
     def description(self, new_description: str) -> None:
-        """
-        Args:
-            new_description: description to set for project.
-        """
         self.gitlab_repo.description = new_description
         self.gitlab_repo.save()
 
     def get_fork(self, create: bool = True) -> Optional["GitlabProject"]:
-        """
-        Provide GitlabProject instance of a fork of this project.
-
-        Returns None if this is a fork.
-
-        :param create: create a fork if it doesn't exist
-        :return: instance of GitlabProject
-        """
         username = self.service.user.get_username()
         for fork in self.get_forks():
             if fork.gitlab_repo.namespace["full_path"] == username:
@@ -172,17 +153,17 @@ class GitlabProject(BaseGitProject):
 
     def get_owners(self) -> List[str]:
         return self._get_collaborators_with_given_access(
-            access_levels=[gitlab.OWNER_ACCESS]
+            access_levels=[gitlab.const.OWNER_ACCESS]
         )
 
     def who_can_close_issue(self) -> Set[str]:
         return set(
             self._get_collaborators_with_given_access(
                 access_levels=[
-                    gitlab.REPORTER_ACCESS,
-                    gitlab.DEVELOPER_ACCESS,
-                    gitlab.MAINTAINER_ACCESS,
-                    gitlab.OWNER_ACCESS,
+                    gitlab.const.REPORTER_ACCESS,
+                    gitlab.const.DEVELOPER_ACCESS,
+                    gitlab.const.MAINTAINER_ACCESS,
+                    gitlab.const.OWNER_ACCESS,
                 ]
             )
         )
@@ -191,9 +172,9 @@ class GitlabProject(BaseGitProject):
         return set(
             self._get_collaborators_with_given_access(
                 access_levels=[
-                    gitlab.DEVELOPER_ACCESS,
-                    gitlab.MAINTAINER_ACCESS,
-                    gitlab.OWNER_ACCESS,
+                    gitlab.const.DEVELOPER_ACCESS,
+                    gitlab.const.MAINTAINER_ACCESS,
+                    gitlab.const.OWNER_ACCESS,
                 ]
             )
         )
@@ -215,10 +196,19 @@ class GitlabProject(BaseGitProject):
             30 => Developer access
             40 => Maintainer access
             50 => Owner access
-        :return: List of usernames
+
+        Returns:
+            List of usernames.
         """
+        # TODO: Remove once ‹members_all› is available for all releases of ogr
+        all_members = None
+        if hasattr(self.gitlab_repo, "members_all"):
+            all_members = self.gitlab_repo.members_all.list(all=True)
+        else:
+            all_members = self.gitlab_repo.members.all(all=True)
+
         response = []
-        for member in self.gitlab_repo.members.all(all=True):
+        for member in all_members:
             if isinstance(member, dict):
                 access_level = member["access_level"]
                 username = member["username"]
@@ -230,36 +220,29 @@ class GitlabProject(BaseGitProject):
         return response
 
     def add_user(self, user: str, access_level: AccessLevel) -> None:
-        """
-        AccessLevel.pull => Guest access
-        AccessLevel.triage => Reporter access
-        AccessLevel.push => Developer access
-        AccessLevel.admin => Maintainer access
-        AccessLevel.maintain => Owner access # Only valid for groups
-        """
         access_dict = {
-            AccessLevel.pull: gitlab.GUEST_ACCESS,
-            AccessLevel.triage: gitlab.REPORTER_ACCESS,
-            AccessLevel.push: gitlab.DEVELOPER_ACCESS,
-            AccessLevel.admin: gitlab.MAINTAINER_ACCESS,
-            AccessLevel.maintain: gitlab.OWNER_ACCESS,
+            AccessLevel.pull: gitlab.const.GUEST_ACCESS,
+            AccessLevel.triage: gitlab.const.REPORTER_ACCESS,
+            AccessLevel.push: gitlab.const.DEVELOPER_ACCESS,
+            AccessLevel.admin: gitlab.const.MAINTAINER_ACCESS,
+            AccessLevel.maintain: gitlab.const.OWNER_ACCESS,
         }
         try:
             user_id = self.service.gitlab_instance.users.list(username=user)[0].id
         except Exception as e:
-            raise GitlabAPIException(f"User {user} not found", e)
+            raise GitlabAPIException(f"User {user} not found") from e
         try:
             self.gitlab_repo.members.create(
                 {"user_id": user_id, "access_level": access_dict[access_level]}
             )
         except Exception as e:
-            raise GitlabAPIException(f"User {user} already exists", e)
+            raise GitlabAPIException(f"User {user} already exists") from e
 
     def request_access(self) -> None:
         try:
             self.gitlab_repo.accessrequests.create({})
         except gitlab.exceptions.GitlabCreateError as e:
-            raise GitlabAPIException("Unable to request access", e)
+            raise GitlabAPIException("Unable to request access") from e
 
     @indirect(GitlabPullRequest.get_list)
     def get_pr_list(self, status: PRStatus = PRStatus.open) -> List["PullRequest"]:
@@ -271,7 +254,7 @@ class GitlabProject(BaseGitProject):
             return tag.attributes["commit"]["id"]
         except gitlab.exceptions.GitlabGetError as ex:
             logger.error(f"Tag {tag_name} was not found.")
-            raise GitlabAPIException(f"Tag {tag_name} was not found.", ex)
+            raise GitlabAPIException(f"Tag {tag_name} was not found.") from ex
 
     @indirect(GitlabPullRequest.create)
     def create_pr(
@@ -287,20 +270,11 @@ class GitlabProject(BaseGitProject):
     def commit_comment(
         self, commit: str, body: str, filename: str = None, row: int = None
     ) -> "CommitComment":
-        """
-        Create comment on a commit.
-
-        :param commit: str The SHA of the commit needing a comment.
-        :param body: str The text of the comment
-        :param filename: str The relative path to the file that necessitates a comment
-        :param row: int Line index in the diff to comment on.
-        :return: CommitComment
-        """
         try:
             commit_object = self.gitlab_repo.commits.get(commit)
-        except gitlab.exceptions.GitlabGetError:
+        except gitlab.exceptions.GitlabGetError as ex:
             logger.error(f"Commit {commit} was not found.")
-            raise GitlabAPIException(f"Commit {commit} was not found.")
+            raise GitlabAPIException(f"Commit {commit} was not found.") from ex
 
         if filename and row:
             raw_comment = commit_object.comments.create(
@@ -320,26 +294,10 @@ class GitlabProject(BaseGitProject):
         context: str,
         trim: bool = False,
     ) -> "CommitFlag":
-        """
-        Create a status on a commit
-
-        :param commit: The SHA of the commit.
-        :param state: The state of the status.
-        :param target_url: The target URL to associate with this status.
-        :param description: A short description of the status
-        :param context: A label to differentiate this status from the status of other systems.
-        :param trim: Whether to trim the description to 140 characters
-        :return: CommitFlag
-        """
         pass
 
     @indirect(GitlabCommitFlag.get)
     def get_commit_statuses(self, commit: str) -> List[CommitFlag]:
-        """
-        Get the statuses of a commit in a project.
-        :param commit: The SHA of the commit.
-        :return: [CommitFlag]
-        """
         pass
 
     def get_git_urls(self) -> Dict[str, str]:
@@ -348,18 +306,19 @@ class GitlabProject(BaseGitProject):
             "ssh": self.gitlab_repo.attributes["ssh_url_to_repo"],
         }
 
-    def fork_create(self) -> "GitlabProject":
-        """
-        Fork this project using the authenticated user.
-        This may raise an exception if the fork already exists.
+    def fork_create(self, namespace: Optional[str] = None) -> "GitlabProject":
+        data = {}
+        if namespace:
+            data["namespace_path"] = namespace
 
-        :return: fork GitlabProject instance
-        """
         try:
-            fork = self.gitlab_repo.forks.create({})
-        except gitlab.GitlabCreateError:
+            fork = self.gitlab_repo.forks.create(data=data)
+        except gitlab.GitlabCreateError as ex:
             logger.error(f"Repo {self.gitlab_repo} cannot be forked")
-            raise GitlabAPIException(f"Repo {self.gitlab_repo} cannot be forked")
+            raise GitlabAPIException(
+                f"Repo {self.gitlab_repo} cannot be forked"
+            ) from ex
+        logger.debug(f"Forked to {fork.namespace['full_path']}/{fork.path}")
         return GitlabProject(
             namespace=fork.namespace["full_path"], service=self.service, repo=fork.path
         )
@@ -377,19 +336,12 @@ class GitlabProject(BaseGitProject):
             return file.decode().decode()
         except gitlab.exceptions.GitlabGetError as ex:
             if ex.response_code == 404:
-                raise FileNotFoundError(f"File '{path}' on {ref} not found", ex)
-            raise GitlabAPIException(ex)
+                raise FileNotFoundError(f"File '{path}' on {ref} not found") from ex
+            raise GitlabAPIException() from ex
 
     def get_files(
         self, ref: str = None, filter_regex: str = None, recursive: bool = False
     ) -> List[str]:
-        """
-        Get a list of file paths of the repo.
-        :param ref: branch or commit (defaults to repo's default branch)
-        :param filter_regex: filter the paths with re.search
-        :param recursive: whether to return only top directory files or all files recursively
-        :return: [str]
-        """
         ref = ref or self.default_branch
         paths = [
             file_dict["path"]
@@ -440,71 +392,43 @@ class GitlabProject(BaseGitProject):
         git_tag = self.gitlab_repo.tags.get(tag_name)
         return GitTag(name=git_tag.name, commit_sha=git_tag.commit["id"])
 
+    @indirect(GitlabRelease.get_list)
     def get_releases(self) -> List[Release]:
-        if not hasattr(self.gitlab_repo, "releases"):
-            raise OperationNotSupported(
-                "This version of python-gitlab does not support release, please upgrade."
-            )
-        releases = self.gitlab_repo.releases.list(all=True)
-        return [
-            self._release_from_gitlab_object(
-                raw_release=release,
-                git_tag=self._git_tag_from_tag_name(release.tag_name),
-            )
-            for release in releases
-        ]
+        pass
 
+    @indirect(GitlabRelease.get)
     def get_release(self, identifier=None, name=None, tag_name=None) -> GitlabRelease:
-        release = self.gitlab_repo.releases.get(tag_name)
-        return self._release_from_gitlab_object(
-            raw_release=release, git_tag=self._git_tag_from_tag_name(release.tag_name)
-        )
+        pass
 
+    @indirect(GitlabRelease.create)
     def create_release(
-        self, name: str, tag_name: str, description: str, ref=None
+        self, tag: str, name: str, message: str, commit_sha: Optional[str] = None
     ) -> GitlabRelease:
-        release = self.gitlab_repo.releases.create(
-            {"name": name, "tag_name": tag_name, "description": description, "ref": ref}
-        )
-        return self._release_from_gitlab_object(
-            raw_release=release, git_tag=self._git_tag_from_tag_name(release.tag_name)
-        )
+        pass
 
+    @indirect(GitlabRelease.get_latest)
     def get_latest_release(self) -> Optional[GitlabRelease]:
-        releases = self.gitlab_repo.releases.list()
-        # list of releases sorted by released_at
-        return (
-            self._release_from_gitlab_object(
-                raw_release=releases[0],
-                git_tag=self._git_tag_from_tag_name(releases[0].tag_name),
-            )
-            if releases
-            else None
-        )
+        pass
 
     def list_labels(self):
         """
-        TODO: Not in API yet.
         Get list of labels in the repository.
-        :return: [Label]
+
+        Returns:
+            List of labels in the repository.
         """
         return list(self.gitlab_repo.labels.list())
 
     def get_forks(self) -> List["GitlabProject"]:
-        """
-        Get forks of the project.
-
-        :return: [GitlabProject]
-        """
         try:
             forks = self.gitlab_repo.forks.list()
-        except KeyError:
+        except KeyError as ex:
             # > item = self._data[self._current]
             # > KeyError: 0
             # looks like some API weirdness
             raise OperationNotSupported(
                 "Please upgrade python-gitlab to a newer version."
-            )
+            ) from ex
         return [
             GitlabProject(
                 repo=fork.path,
@@ -516,11 +440,13 @@ class GitlabProject(BaseGitProject):
 
     def update_labels(self, labels):
         """
-        TODO: Not in API yet.
         Update the labels of the repository. (No deletion, only add not existing ones.)
 
-        :param labels: [str]
-        :return: int - number of added labels
+        Args:
+            labels: List of labels to be added.
+
+        Returns:
+            Number of added labels.
         """
         current_label_names = [la.name for la in list(self.gitlab_repo.labels.list())]
         changes = 0
@@ -550,23 +476,26 @@ class GitlabProject(BaseGitProject):
             sha=commit, comment=raw_comment.note, author=raw_comment.author["username"]
         )
 
-    def _release_from_gitlab_object(
-        self, raw_release, git_tag: GitTag
-    ) -> GitlabRelease:
-        return GitlabRelease(
-            tag_name=raw_release.tag_name,
-            url=None,
-            created_at=raw_release.created_at,
-            tarball_url=raw_release.assets["sources"][1]["url"],
-            git_tag=git_tag,
-            project=self,
-            raw_release=raw_release,
-        )
-
     def get_web_url(self) -> str:
-        """
-        Get web URL of the project.
-
-        :return: str
-        """
         return self.gitlab_repo.web_url
+
+    def get_sha_from_branch(self, branch: str) -> Optional[str]:
+        try:
+            return self.gitlab_repo.branches.get(branch).attributes["commit"]["id"]
+        except GitlabGetError as ex:
+            if ex.response_code == 404:
+                return None
+            raise GitlabAPIException from ex
+
+    def get_contributors(self) -> Set[str]:
+        """
+        Returns:
+            Unique authors of the commits in the project.
+        """
+
+        def format_contributor(contributor: Dict[str, Any]) -> str:
+            return f"{contributor['name']} <{contributor['email']}>"
+
+        return set(
+            map(format_contributor, self.gitlab_repo.repository_contributors(all=True))
+        )
