@@ -1,24 +1,5 @@
-# MIT License
-#
-# Copyright (c) 2018-2019 Red Hat, Inc.
-
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in all
-# copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
+# Copyright Contributors to the Packit project.
+# SPDX-License-Identifier: MIT
 
 import logging
 from typing import List, Optional, Dict, Set, Iterable
@@ -102,7 +83,7 @@ class PagureProject(BaseGitProject):
         self,
         *args,
         add_fork_part: bool = True,
-        add_api_endpoint_part=True,
+        add_api_endpoint_part: bool = True,
         method: str = None,
         params: dict = None,
         data: dict = None,
@@ -110,13 +91,20 @@ class PagureProject(BaseGitProject):
         """
         Call project API endpoint.
 
-        :param args: str parts of the url (e.g. "a", "b" will call "project/a/b")
-        :param add_fork_part: If the projects is a fork, use "fork/username" prefix, True by default
-        :param add_api_endpoint_part: Add part with API endpoint "/api/0/"
-        :param method: "GET"/"POST"/...
-        :param params: http(s) query parameters
-        :param data: data to be sent
-        :return: dict
+        Args:
+            *args: String parts of the URL, e.g. `"a", "b"` will call `project/a/b`
+            add_fork_part: If the project is a fork, use `fork/username` prefix.
+
+                Defaults to `True`.
+            add_api_endpoint_part: Add part with API endpoint (`/api/0/`).
+
+                Defaults to `True`.
+            method: Method of the HTTP request, e.g. `"GET"`, `"POST"`, etc.
+            params: HTTP(S) query parameters in form of a dictionary.
+            data: Data to be sent in form of a dictionary.
+
+        Returns:
+            Dictionary representing response.
         """
         request_url = self._get_project_url(
             *args,
@@ -132,7 +120,7 @@ class PagureProject(BaseGitProject):
         self,
         *args,
         add_fork_part: bool = True,
-        add_api_endpoint_part=True,
+        add_api_endpoint_part: bool = True,
         method: str = None,
         params: dict = None,
         data: dict = None,
@@ -140,13 +128,20 @@ class PagureProject(BaseGitProject):
         """
         Call project API endpoint.
 
-        :param args: str parts of the url (e.g. "a", "b" will call "project/a/b")
-        :param add_fork_part: If the projects is a fork, use "fork/username" prefix, True by default
-        :param add_api_endpoint_part: Add part with API endpoint "/api/0/"
-        :param method: "GET"/"POST"/...
-        :param params: http(s) query parameters
-        :param data: data to be sent
-        :return: RequestResponse
+        Args:
+            *args: String parts of the URL, e.g. `"a", "b"` will call `project/a/b`
+            add_fork_part: If the project is a fork, use `fork/username` prefix.
+
+                Defaults to `True`.
+            add_api_endpoint_part: Add part with API endpoint (`/api/0/`).
+
+                Defaults to `True`.
+            method: Method of the HTTP request, e.g. `"GET"`, `"POST"`, etc.
+            params: HTTP(S) query parameters in form of a dictionary.
+            data: Data to be sent in form of a dictionary.
+
+        Returns:
+            `RequestResponse` object containing response.
         """
         request_url = self._get_project_url(
             *args,
@@ -187,20 +182,17 @@ class PagureProject(BaseGitProject):
 
     @property
     def description(self) -> str:
-        """
-        Returns:
-            Project description.
-        """
         return self.get_project_info()["description"]
 
     @description.setter
     def description(self, new_description: str) -> None:
-        """
-        Args:
-            new_description: description to set for project.
-        """
         raise OperationNotSupported("Not possible on Pagure")
 
+    @property
+    def has_issues(self) -> bool:
+        options = self._call_project_api("options", method="GET")
+        return options["settings"]["issue_tracker"]
+
     def get_owners(self) -> List[str]:
         project = self.get_project_info()
         return project["access_users"]["owner"]
@@ -279,14 +271,21 @@ class PagureProject(BaseGitProject):
         pass
 
     @if_readonly(return_function=GitProjectReadOnly.fork_create)
-    def fork_create(self) -> "PagureProject":
+    def fork_create(self, namespace: Optional[str] = None) -> "PagureProject":
+        if namespace is not None:
+            raise OperationNotSupported(
+                "Pagure does not support forking to namespaces."
+            )
+
         request_url = self.service.get_api_url("fork")
         self.service.call_api(
             url=request_url,
             method="POST",
             data={"repo": self.repo, "namespace": self.namespace, "wait": True},
         )
-        return self._construct_fork_project()
+        fork = self._construct_fork_project()
+        logger.debug(f"Forked to {fork.full_repo_name}")
+        return fork
 
     def _construct_fork_project(self) -> "PagureProject":
         return PagureProject(
@@ -298,14 +297,6 @@ class PagureProject(BaseGitProject):
         )
 
     def get_fork(self, create: bool = True) -> Optional["PagureProject"]:
-        """
-        Provide GitProject instance of a fork of this project.
-
-        Returns None if this is a fork.
-
-        :param create: create a fork if it doesn't exist
-        :return: instance of GitProject or None
-        """
         if self.is_fork:
             raise OgrException("Cannot create fork from fork.")
 
@@ -330,11 +321,6 @@ class PagureProject(BaseGitProject):
         return response.ok
 
     def is_private(self) -> bool:
-        """
-        Is this repo private? (accessible only by users with granted access)
-
-        :return: if yes, return True
-        """
         host = urlparse(self.service.instance_url).hostname
         if host in [
             "git.centos.org",
@@ -351,11 +337,6 @@ class PagureProject(BaseGitProject):
         )
 
     def is_forked(self) -> bool:
-        """
-        Is this repo forked by the authenticated user?
-
-        :return: if yes, return True
-        """
         f = self._construct_fork_project()
         return bool(f.exists() and f.parent.exists())
 
@@ -368,9 +349,6 @@ class PagureProject(BaseGitProject):
 
     @property
     def parent(self) -> Optional["PagureProject"]:
-        """
-        Return parent project if this project is a fork, otherwise return None
-        """
         if self.get_is_fork_from_api():
             return PagureProject(
                 repo=self.repo,
@@ -384,23 +362,9 @@ class PagureProject(BaseGitProject):
         return return_value["urls"]
 
     def add_user(self, user: str, access_level: AccessLevel) -> None:
-        """
-        AccessLevel.pull => ticket
-        AccessLevel.triage => ticket
-        AccessLevel.push => commit
-        AccessLevel.admin => commit
-        AccessLevel.maintain => admin
-        """
         self.add_user_or_group(user, access_level, "user")
 
     def add_group(self, group: str, access_level: AccessLevel):
-        """
-        AccessLevel.pull => ticket
-        AccessLevel.triage => ticket
-        AccessLevel.push => commit
-        AccessLevel.admin => commit
-        AccessLevel.maintain => admin
-        """
         self.add_user_or_group(group, access_level, "group")
 
     def add_user_or_group(
@@ -425,14 +389,12 @@ class PagureProject(BaseGitProject):
         )
 
         if response.status_code == 401:
-            raise PagureAPIException("You are not allowed to modify ACL's")
+            raise PagureAPIException(
+                "You are not allowed to modify ACL's",
+                response_code=response.status_code,
+            )
 
     def change_token(self, new_token: str) -> None:
-        """
-        Change an API token.
-
-        Only for this instance.
-        """
         self.service.change_token(new_token)
 
     def get_file_content(self, path: str, ref=None) -> str:
@@ -448,7 +410,7 @@ class PagureProject(BaseGitProject):
     def get_sha_from_tag(self, tag_name: str) -> str:
         tags_dict = self.get_tags_dict()
         if tag_name not in tags_dict:
-            raise PagureAPIException(f"Tag '{tag_name}' not found.")
+            raise PagureAPIException(f"Tag '{tag_name}' not found.", response_code=404)
 
         return tags_dict[tag_name].commit_sha
 
@@ -484,33 +446,25 @@ class PagureProject(BaseGitProject):
         response = self._call_project_api("git", "tags", params={"with_commits": True})
         return {n: GitTag(name=n, commit_sha=c) for n, c in response["tags"].items()}
 
+    @indirect(PagureRelease.get_list)
     def get_releases(self) -> List[Release]:
-        # git tag for Pagure is shown as Release in Pagure UI
-        git_tags = self.get_tags()
-        return [self._release_from_git_tag(git_tag) for git_tag in git_tags]
+        pass
 
+    @indirect(PagureRelease.get)
     def get_release(self, identifier=None, name=None, tag_name=None) -> PagureRelease:
-        raise OperationNotSupported
+        pass
 
+    @indirect(PagureRelease.get_latest)
     def get_latest_release(self) -> Optional[PagureRelease]:
-        raise OperationNotSupported
+        pass
 
-    def _release_from_git_tag(self, git_tag: GitTag) -> PagureRelease:
-        return PagureRelease(
-            tag_name=git_tag.name,
-            url="",
-            created_at="",
-            tarball_url="",
-            git_tag=git_tag,
-            project=self,
-        )
+    @indirect(PagureRelease.create)
+    def create_release(
+        self, tag: str, name: str, message: str, ref: Optional[str] = None
+    ) -> Release:
+        pass
 
     def get_forks(self) -> List["PagureProject"]:
-        """
-        Get forks of the project.
-
-        :return: [PagureProject]
-        """
         forks_url = self.service.get_api_url("projects")
         projects_response = self.service.call_api(
             url=forks_url, params={"fork": True, "pattern": self.repo}
@@ -527,21 +481,10 @@ class PagureProject(BaseGitProject):
         ]
 
     def get_web_url(self) -> str:
-        """
-        Get web URL of the project.
-
-        :return: str
-        """
         return f'{self.service.instance_url}/{self.get_project_info()["url_path"]}'
 
     @property
     def full_repo_name(self) -> str:
-        """
-        Get repo name with namespace
-        e.g. 'rpms/python-docker-py'
-
-        :return: str
-        """
         fork = f"fork/{self._user}/" if self.is_fork else ""
         namespace = f"{self.namespace}/" if self.namespace else ""
         return f"{fork}{namespace}{self.repo}"
@@ -573,3 +516,13 @@ class PagureProject(BaseGitProject):
             paths = filter_paths(paths, filter_regex)
 
         return paths
+
+    def get_sha_from_branch(self, branch: str) -> Optional[str]:
+        branches = self._call_project_api(
+            "git", "branches", params={"with_commits": True}
+        )["branches"]
+
+        return branches.get(branch)
+
+    def get_contributors(self) -> Set[str]:
+        raise OperationNotSupported("Pagure doesn't provide list of contributors")
