@@ -1,24 +1,5 @@
-# MIT License
-#
-# Copyright (c) 2018-2019 Red Hat, Inc.
-
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in all
-# copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
+# Copyright Contributors to the Packit project.
+# SPDX-License-Identifier: MIT
 
 import datetime
 import logging
@@ -116,7 +97,8 @@ class PagurePullRequest(BasePullRequest):
         )
         if request_response.status_code != 200:
             raise PagureAPIException(
-                f"Cannot get patch from {self.url}.patch because {request_response.reason}."
+                f"Cannot get patch from {self.url}.patch because {request_response.reason}.",
+                response_code=request_response.status_code,
             )
         return request_response.content
 
@@ -227,7 +209,7 @@ class PagurePullRequest(BasePullRequest):
             self._raw_pr = updated_pr
             return self
         except Exception as ex:
-            raise PagureAPIException("there was an error while updating the PR", ex)
+            raise PagureAPIException("there was an error while updating the PR") from ex
 
     def _get_all_comments(self) -> List[PRComment]:
         self.__update()
@@ -255,7 +237,9 @@ class PagurePullRequest(BasePullRequest):
         self.__call_api("comment", method="POST", data=payload)
         self.__dirty = True
         return PagurePRComment(
-            parent=self, body=body, author=self.project.service.user.get_username()
+            parent=self,
+            body=body,
+            author=self.target_project.service.user.get_username(),
         )
 
     def close(self) -> "PullRequest":
@@ -292,20 +276,20 @@ class PagurePullRequest(BasePullRequest):
         """
         Set a flag on a pull-request to display results or status of CI tasks.
 
-        See "Flag a pull-request" at
-
-            https://pagure.io/api/0/#pull_requests-tab
-
+        See "Flag a pull-request" at https://pagure.io/api/0/#pull_requests-tab
         for a full description of the parameters.
 
-        :param username: The name of the application to be presented to users
-                         on the pull request page.
-        :param comment: A short message summarizing the presented results.
-        :param url: A URL to the result of this flag.
-        :param status: The status to be displayed for this flag.
-        :param percent: A percentage of completion compared to the goal.
-        :param uid: A unique identifier used to identify a flag on the pull-request.
-        :return: dict with the response received from Pagure.
+        Args:
+            username: The name of the application to be presented to users
+                on the pull request page.
+            comment: A short message summarizing the presented results.
+            url: A URL to the result of this flag.
+            status: The status to be displayed for this flag.
+            percent: A percentage of completion compared to the goal.
+            uid: A unique identifier used to identify a flag on the pull-request.
+
+        Returns:
+            Dictionary with the response received from Pagure.
         """
         data: Dict[str, Union[str, int]] = {
             "username": username,
@@ -319,3 +303,12 @@ class PagurePullRequest(BasePullRequest):
         if uid is not None:
             data["uid"] = uid
         return self.__call_api("flag", method="POST", data=data)
+
+    def get_comment(self, comment_id: int) -> PRComment:
+        for comment in self._get_all_comments():
+            if comment.id == comment_id:
+                return comment
+
+        raise PagureAPIException(
+            f"No comment with id#{comment_id} in PR#{self.id} found.", response_code=404
+        )
