@@ -1,3 +1,6 @@
+# Copyright Contributors to the Packit project.
+# SPDX-License-Identifier: MIT
+
 from datetime import datetime
 
 import pytest
@@ -114,6 +117,14 @@ class GenericCommands(GitlabTests):
         assert urls["git"] == "https://gitlab.com/packit-service/ogr-tests.git"
         assert urls["ssh"].endswith("git@gitlab.com:packit-service/ogr-tests.git")
 
+    def test_get_sha_from_branch(self):
+        commit_sha = self.project.get_sha_from_branch("change")
+        assert commit_sha and commit_sha.startswith("d490ec67")
+
+    def test_get_sha_from_branch_non_existing(self):
+        commit_sha = self.project.get_sha_from_branch("non-existing")
+        assert commit_sha is None
+
     def test_get_sha_from_tag(self):
         assert (
             self.project.get_sha_from_tag("0.1.0")
@@ -222,3 +233,15 @@ class GenericCommands(GitlabTests):
             repo="delete-project", namespace="shreyaspapi"
         )
         project.delete()
+
+    def test_has_issues(self):
+        assert self.project.has_issues
+        assert not self.service.get_project(
+            namespace="redhat/centos-stream/rpms", repo="firefox"
+        ).has_issues
+
+    def test_get_contributors(self):
+        project = self.service.get_project(namespace="recalbox", repo="recalbox")
+        owners = project.get_owners()
+        contributors = project.get_contributors()
+        assert len(owners) < len(contributors)
