@@ -1,4 +1,8 @@
+# Copyright Contributors to the Packit project.
+# SPDX-License-Identifier: MIT
+
 from requre.online_replacing import record_requests_for_all_methods
+from ogr.exceptions import IssueTrackerDisabled
 
 from tests.integration.pagure.base import PagureTests
 from ogr.abstract import IssueStatus
@@ -107,3 +111,16 @@ class Issues(PagureTests):
         issue = project.create_issue(title=title, body=description)
         assert issue.title == title
         assert issue.description == description
+
+    def test_get_comment(self):
+        project = self.service.get_project(
+            repo="my-playground", namespace=None, username="nikromen"
+        )
+        comment = project.get_issue(1).get_comment(753462)
+        assert comment.body == "example issue comment"
+
+    def test_create_with_disabled_issues(self):
+        with self.assertRaises(IssueTrackerDisabled):
+            self.ogr_project.get_fork().create_issue(
+                "Testing issue", "shouldn't be created"
+            )
