@@ -1,3 +1,6 @@
+# Copyright Contributors to the Packit project.
+# SPDX-License-Identifier: MIT
+
 from requre.online_replacing import record_requests_for_all_methods
 
 from tests.integration.pagure.base import PagureTests
@@ -78,6 +81,12 @@ class PullRequests(PagureTests):
             == "517121273b142293807606dbd7a2e0f514b21cc8"
         )
 
+    def test_target_branch_head_commit(self):
+        assert (
+            self.ogr_project.get_pr(13).target_branch_head_commit
+            == "8d11eac1773ee032977f5ec4a86683113d6a888d"
+        )
+
     def test_source_project_upstream_branch(self):
         pr = self.ogr_project.get_pr(4)
         source_project = pr.source_project
@@ -102,3 +111,7 @@ class PullRequests(PagureTests):
         assert (
             pr.commits_url == "https://pagure.io/ogr-tests/pull-request/5#commit_list"
         )
+
+    def test_get_comment(self):
+        comment = self.ogr_project.get_pr(5).get_comment(comment_id=162373)
+        assert "Billy: what the hell is going on up there?" in comment.body
